@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
 NoScript - a Firefox extension for whitelist driven safe JavaScript execution
-Copyright (C) 2004-2007 Giorgio Maone - g.maone@informaction.com
+Copyright (C) 2004-2008 Giorgio Maone - g.maone@informaction.com
 
 Contributors: 
   Higmmer
@@ -49,9 +49,13 @@ var noscriptOverlay = noscriptUtil.service ?
   
   get prompter() {
     return noscriptUtil.prompter;
-  }
+  },
+  
+  onMenuShowing: function(ev) {
+    if (ev.currentTarget != ev.originalTarget) return;
+    this.prepareMenu(ev.target);
+  },
 
-,
   prepareContextMenu: function(ev) {
     var menu = document.getElementById("noscript-context-menu");
     if (this.ns.getPref("ctxMenu", true)) {
@@ -140,15 +144,13 @@ var noscriptOverlay = noscriptUtil.service ?
     const miGlobal = seps.global.nextSibling;
     
     if (global || ns.getPref("showGlobal")) {
-      miGlobal.style.display = "";
-      seps.global.style.display = "";
+      miGlobal.hidden = seps.global.hidden = false;
       miGlobal.setAttribute("label", this.getString((global ? "forbid" : "allow") + "Global"));
       miGlobal.setAttribute("oncommand","noscriptOverlay.menuAllow("+(!global)+")");
       miGlobal.setAttribute("tooltiptext", this.statusIcon.getAttribute("tooltiptext"));
-      miGlobal.setAttribute("class", "menuitem-iconic " + this.getStatusClass(global ? "no" : "glb"));
+      miGlobal.setAttribute("class", "menuitem-iconic noscript-glb " + this.getStatusClass(global ? "no" : "yes"));
     } else {
-      miGlobal.style.display = "none";
-      seps.global.style.display = "none";
+      miGlobal.hidden = seps.global.hidden = true;
     }
     
     node = miGlobal.nextSibling;
@@ -158,18 +160,21 @@ var noscriptOverlay = noscriptUtil.service ?
       node = mainMenu.insertBefore(document.getElementById("noscript-revoke-temp-mi").cloneNode(true), node);
       node.id = "";
     }
-    if (ns.tempSites.sitesString) {
-      node.style.display = "";
-      seps.global.style.display = "";
+    if ((ns.tempSites.sitesString || ns.objectWhitelistLen) && ns.getPref("showRevokeTemp", true)) {
+      node.hidden = seps.global.hidden = false;
       node.setAttribute("tooltiptext",
         // remove http/https/file CAPS hack entries 
-        ns.siteUtils.sanitizeString(ns.tempSites.sitesString.replace(/\b(?:https?|file):\/\//g, ""))
-          .split(/\s+/) 
-          // .join(",\n") // wait Firefox 3 with its multiline tooltips for this
-          .length
-        );
+        "<SCRIPT>: " + 
+        (ns.tempSites.sitesString 
+          ? ns.siteUtils.sanitizeString(ns.tempSites.sitesString.replace(/\b(?:https?|file):\/\//g, ""
+              )).split(/\s+/) 
+              // .join(",\n") // wait Firefox 3 with its multiline tooltips for this
+              .length
+          : 0) +
+        " | <OBJECT>: " + ns.objectWhitelistLen
+      );
     } else {
-      node.style.display = "none";
+      node.hidden = true;
     }
     node = node.nextSibling;
    
@@ -186,18 +191,25 @@ var noscriptOverlay = noscriptUtil.service ?
     this.prepareOptItems(popup);
       
     var untrustedMenu = null;
+    var pluginsMenu = null;
     if (seps.untrusted) {
-      node = document.getElementById("noscript-menu-untrusted");
-      if (seps.untrusted.nextSibling != node) {
-        seps.untrusted.parentNode.insertBefore(node, seps.untrusted.nextSibling);
-      }
       
-      untrustedMenu = node.firstChild;
-      while (untrustedMenu.firstChild) {
-        untrustedMenu.removeChild(untrustedMenu.firstChild);
+      pluginsMenu = document.getElementById("noscript-menu-blocked-objects");
+      untrustedMenu = document.getElementById("noscript-menu-untrusted");
+      // cleanup untrustedCount display
+      untrustedMenu.setAttribute("label", untrustedMenu.getAttribute("label").replace(/ \(\d+\)$/, ""));
+      
+      with(seps.untrusted) {
+        if (nextSibling != pluginsMenu) {   
+          parentNode.insertBefore(untrustedMenu, nextSibling);
+          parentNode.insertBefore(pluginsMenu, untrustedMenu);
+        }
       }
+      // descend from menus to popups and clear children
+      for each(node in [pluginsMenu = pluginsMenu.firstChild, untrustedMenu = untrustedMenu.firstChild])
+        while (node.firstChild)
+          node.removeChild(node.firstChild);
       untrustedMenu.appendCmd = untrustedMenu.appendChild;
-      this.updateStatusClass(untrustedMenu.parentNode, this.getStatusClass("no", true));
     }
     
     node = seps.insert.nextSibling;
@@ -213,7 +225,11 @@ var noscriptOverlay = noscriptUtil.service ?
     mainMenu.appendCmd = function(n) { this.insertBefore(n, seps.stop); };
 
     const sites = this.getSites();
-    
+    try {
+      this.populatePluginsMenu(mainMenu, pluginsMenu, sites.pluginExtras);
+    } catch(e) {
+      if(ns.consoleDump) ns.dump("Error populating plugins menu: " + e);
+    }
     var site, enabled, isTop, lev;
     var jsPSs = ns.jsPolicySites;
     var matchingSite;
@@ -242,9 +258,7 @@ var noscriptOverlay = noscriptUtil.service ?
     const showTemp = !locked && ns.getPref("showTemp", true);
     
     var parent, extraNode;
-    
-   
-    
+    var untrustedCount = 0;
     
     for (j = sites.length; j-->0;) {
       site = sites[j];
@@ -298,7 +312,9 @@ var noscriptOverlay = noscriptUtil.service ?
       for (scount = menuSites.length; scount-- > 0;) {
         menuSite = menuSites[scount];
         
-        untrusted = (!enabled) && ns.isUntrusted(menuSite);
+        if (untrusted = (!enabled) && ns.isUntrusted(menuSite)) 
+          untrustedCount++;
+        
         parent = (untrusted && showUntrusted) ? untrustedMenu : mainMenu;
         if (!parent) continue;
         
@@ -341,9 +357,14 @@ var noscriptOverlay = noscriptUtil.service ?
           }
         }
       }
-      if (untrustedMenu && untrustedMenu.firstChild) {
-        untrustedMenu.appendCmd(document.createElement("menuseparator"));
-      }
+      
+    }
+    if (untrustedMenu && untrustedMenu.firstChild) {
+      untrustedMenu.appendCmd(document.createElement("menuseparator"));
+      if (untrustedCount) 
+        with(untrustedMenu.parentNode)
+          setAttribute("label", getAttribute("label") +
+            " (" + untrustedCount + ")"); // see above for cleanup
     }
     this.normalizeMenu(untrustedMenu, true);
     this.normalizeMenu(mainMenu, false);
@@ -352,27 +373,122 @@ var noscriptOverlay = noscriptUtil.service ?
       // this one can go away, better take our stuff back when done
       mainMenu.addEventListener("popuphidden", function(ev) {
         if (ev.target != ev.currentTarget) return;
-        ev.currentTarget.removeEventListener(ev.name, arguments.callee, false);
+        ev.currentTarget.removeEventListener(ev.type, arguments.callee, false);
         noscriptOverlay.prepareMenu(document.getElementById("noscript-status-popup"));
       }, false);
     }
     
   },
+
+  populatePluginsMenu: function(mainMenu, menu, extras) {
+    if (!menu) return;
+    
+    menu.parentNode.hidden = true;
+    const ns = this.ns;
+    if (!(extras && ns.getPref("showBlockedObjects", true)))
+      return;
+    
+    var egroup, e, node, i;
+    var pluginExtras = [];
+    i = 0;
+    for each(egroup in extras) {
+      for each(e in egroup) {
+         
+         if(e.tag && !e.placeholder) continue;
+         node = document.createElement("menuitem");
+         
+         e.label = e.label || ns.mimeEssentials(e.mime) + "@" + ns.urlEssentials(e.url);
+         e.title = e.title || e.label.split("@")[0] + e.url;
+  
+         node.setAttribute("label", this.getString("allowTemp", [e.label]));
+         node.setAttribute("tooltiptext", e.title);
+         node.setAttribute("oncommand", "noscriptOverlay.allowObject(" + i + ")");
+         node.setAttribute("class", "menuitem-iconic noscript-cmd noscript-temp noscript-allow");
+         node.style.listStyleImage = ns.cssMimeIcon(e.mime, 16);
+         menu.appendChild(node);
+         pluginExtras[i++] = e;
+      }
+    }
+    if (i) {
+      noscriptOverlay.menuPluginExtras = pluginExtras;
+      mainMenu.addEventListener("popuphidden", function(ev) {
+          if (ev.currentTarget != ev.target) return;
+          ev.currentTarget.removeEventListener(ev.type, arguments.callee, false);
+          noscriptOverlay.menuPluginExtras = null;
+          noscriptOverlay.menuPluginSites = null;
+      }, false);
+      var pluginSites = {};
+      i = 0;
+      for each(e in pluginExtras) {
+        if(!(e.site && e.mime)) continue;
+        if (e.site in pluginSites) {
+          if (pluginSites[e.site].indexOf(e.mime) > -1) 
+            continue;
+          pluginSites[e.site].push(e.mime);
+        } else {
+          pluginSites[e.site] = ["*", e.mime];
+        }
+        i++;
+      }
+      if (i) {
+        noscriptOverlay.menuPluginSites = [];
+        i = 0;
+        var mime;
+        for (var site in pluginSites) {
+          menu.appendChild(document.createElement("menuseparator"));
+          for each(mime in pluginSites[site]) {
+            node = document.createElement("menuitem");
+            node.setAttribute("label", this.getString("allowTemp", [ns.mimeEssentials(mime) + "@" + site]));
+            node.setAttribute("tooltiptext", mime + "@" + site);
+            node.setAttribute("oncommand", "noscriptOverlay.allowObjectSite(" + i + ")");
+            node.setAttribute("class", "menuitem-iconic noscript-temp noscript-cmd noscript-allow");
+            if(mime != "*")
+              node.style.listStyleImage = node.style.listStyleImage = ns.cssMimeIcon(mime, 16);
+
+            menu.appendChild(node);
+            noscriptOverlay.menuPluginSites[i++] = [site, mime];
+          }
+        }
+      }
+      menu.parentNode.hidden = false;
+    }
+  },
+  
+  allowObject: function(i) {
+    if(this.menuPluginExtras && this.menuPluginExtras[i]) {
+      var e = this.menuPluginExtras[i];
+      if(e.placeholder) {
+        this.ns.checkAndEnablePlaceholder(e.placeholder);
+      } else if (this.ns.confirmEnableObject(window, e)) {
+        this.allowObjectURL(e.url, e.mime);
+      }
+    }
+  },
+  allowObjectSite: function(i) {
+    if(this.menuPluginSites && this.menuPluginSites[i]) {
+      this.allowObjectURL(this.menuPluginSites[i][0], this.menuPluginSites[i][1]);
+    }
+  },
+  allowObjectURL: function(url, mime) {
+    this.ns.allowObject(url, mime);
+    if (this.ns.getPref("autoReload", true))
+        this.ns.quickReload(gBrowser.selectedBrowser.webNavigation);
+  },
   
   normalizeMenu: function(menu, hideParentIfEmpty) {
     if (!menu) return;
     var prev = null;
-    var wasSep = false;
+    var wasSep = true;
     var isSep, haveMenu = false;
     for (var i = menu.firstChild; i; i = i.nextSibling) {
       if (!i.hidden) {
         isSep = i.nodeName == "menuseparator";
-        if (isSep && (wasSep || !prev)) {
+        if (isSep && wasSep) {
           i.hidden = true;
         } else {
+          haveMenu = haveMenu || !isSep;
           prev = i;
           wasSep = isSep;
-          haveMenu = haveMenu || !isSep;
         }
       }
     }
@@ -389,7 +505,7 @@ var noscriptOverlay = noscriptUtil.service ?
     return browser && browser.contentWindow && browser.contentWindow.document || null;
   }
 ,
-  eraseTemp: function() {
+  revokeTemp: function() {
     noscriptOverlay.ns.safeCapsOp(function() {
       noscriptOverlay.ns.eraseTemp();
       noscriptOverlay.syncUI();
@@ -443,8 +559,8 @@ var noscriptOverlay = noscriptUtil.service ?
     return "noscript-" + (inactive ? "inactive-" : "") + lev;
   },
   updateStatusClass: function(node, className) {
-    if (!className) className = this.statusIcon.className.replace(/.*(\bnoscript-\S*(?:yes|no|glb|prt)).*/, "$1");
-    node.className = (node.className.replace(/\bnoscript-\S*(?:yes|no|glb|prt)\b/g, "") + " " + className).replace(/\s{2,}/g, " ");
+    if (!className) className = this.statusIcon.className.replace(/.*(\bnoscript-\S*(?:yes|no|glb|prt|yu)).*/, "$1");
+    node.className = (node.className.replace(/\bnoscript-\S*(?:yes|no|glb|prt|yu)\b/g, "") + " " + className).replace(/\s{2,}/g, " ");
   }
 ,
   _syncTimeout: null,
@@ -491,7 +607,7 @@ var noscriptOverlay = noscriptUtil.service ?
   },
   
   showUI: function() {
-    var statusIcon = document.getElementById("noscript-statusIcon");
+    var statusIcon = this.statusIcon;
     var popup = document.getElementById("noscript-status-popup");
     if (statusIcon.hidden || statusIcon.parentNode.hidden) {
       var tbb = document.getElementById("noscript-tbb");
@@ -509,7 +625,7 @@ var noscriptOverlay = noscriptUtil.service ?
       } else {
         popup.addEventListener("popuphidden", function(ev) {
             if (ev.currentTarget != ev.target) return;
-            ev.target.removeEventListener("popuphidden", arguments.callee, false);
+            ev.target.removeEventListener(ev.type, arguments.callee, false);
             ev.target.parentNode.hidden = !noscriptOverlay.ns.getPref("statusIcon", true);
         }, false);
         statusIcon.hidden = false;
@@ -690,7 +806,7 @@ var noscriptOverlay = noscriptUtil.service ?
   
   notifyXSSOnLoad: function(requestInfo) {
     requestInfo.browser.addEventListener("DOMContentLoaded", function(ev) {
-      requestInfo.browser.removeEventListener("DOMContentLoaded", arguments.callee, false);
+      requestInfo.browser.removeEventListener(ev.type, arguments.callee, false);
       if (requestInfo.unsafeRequest && requestInfo.unsafeRequest.issued) return;
       noscriptOverlay.notifyXSS(requestInfo);
     }, false);
@@ -767,7 +883,7 @@ var noscriptOverlay = noscriptUtil.service ?
       }
       browser.addEventListener("beforeunload", function(ev) {
         if (ev.originalTarget == info.document || ev.originalTarget == browser) {
-          browser.removeEventListener("beforeunload", arguments.callee, false);
+          browser.removeEventListener(ev.type, arguments.callee, false);
           if (notification && notification == box.currentNotification) {
             box.removeCurrentNotification();
           } else {
@@ -820,7 +936,7 @@ var noscriptOverlay = noscriptUtil.service ?
       );
     browser.addEventListener("beforeunload", function(ev) {
       if (ev.originalTarget == info.document || ev.originalTarget == browser) {
-        browser.removeEventListener("beforeunload", arguments.callee, false);
+        browser.removeEventListener(ev.type, arguments.callee, false);
         if (notification && notification == box.currentNotification) {
           box.removeCurrentNotification();
         } 
@@ -900,6 +1016,7 @@ var noscriptOverlay = noscriptUtil.service ?
     var notificationNeeded = false;
     var allowedSites = [];
     var allowed = 0;
+    var untrusted = 0;
     if (global) {
       lev = "glb";
     } else {
@@ -916,15 +1033,19 @@ var noscriptOverlay = noscriptUtil.service ?
             allowedSites.push(site);
           }
         } else {
-          notificationNeeded = notificationNeeded || url != "about:blank" && !untrustedSites.matches(url);
+          if(!notificationNeeded && url != "about:blank") {
+            if(untrustedSites.matches(url)) untrusted++;
+            else notificationNeeded = true;
+          }
         }
       }
       allowed = allowedSites.length;
-      lev = (allowed == total && sites.length > 0) ? "yes" : allowed == 0 ? "no" : "prt";
+      lev = (allowed == total && sites.length > 0) ? "yes" : allowed == 0 ? "no" 
+            : (untrusted > 0 && !notificationNeeded ? "yu" : "prt");
       notificationNeeded = notificationNeeded && totalAnnoyances > 0;
     }
     
-    var message = this.getString("allowed." + lev);
+    var message = this.getString("allowed." + (lev == "yu" ? "prt" : lev));
     var shortMessage = message.replace(/JavaScript/g, "JS");
     
     if (notificationNeeded && allowed) 
@@ -949,17 +1070,17 @@ var noscriptOverlay = noscriptUtil.service ?
     
     if (notificationNeeded) { // notifications
       const win = window.content;
-      if (this.notify) { 
+      if (this.notify) {
         this.notificationShow(message,
           this.getIcon(widget), 
-          !(win._NoScript_messageShown && this.notifyHidePermanent));
-        win._NoScript_messageShown = true;
+          !(ns.getExpando(win, "messageShown") && this.notifyHidePermanent));
+        ns.setExpando(win, "messageShown", true);
       } else {
         this.notificationHide(); 
       }
-      if (!win._NoScript_soundPlayed) {
+      if (!ns.getExpando(win, "soundPlayed")) {
         ns.soundNotify(window.content.location.href);
-        win._NoScript_soundPlayed = true;
+        ns.setExpando(win, "soundPlayed");
       }
     } else {
       this.notificationHide();
@@ -985,7 +1106,6 @@ var noscriptOverlay = noscriptUtil.service ?
     const ns = this.ns;
     browser = browser || ns.domUtils.findBrowserForNode(doc);
     if (browser) {
-      ns.pluginsCache.purgeURIs(browser);
       ns.setExpando(browser, "pe", null);
       this.notificationHide(browser, true);
     }
@@ -1005,12 +1125,12 @@ var noscriptOverlay = noscriptUtil.service ?
       }
       switch (data) {
         case "statusIcon": case "statusLabel":
-        window.setTimeout(function() {
-            var widget =document.getElementById("noscript-" + data);
-            if (widget) {
-              widget.setAttribute("hidden", !noscriptOverlay.ns.getPref(data))
-            }
-          }, 0);
+          window.setTimeout(function() {
+              var widget =document.getElementById("noscript-" + data);
+              if (widget) {
+                widget.setAttribute("hidden", !noscriptOverlay.ns.getPref(data))
+              }
+            }, 0);
          break;
          case "notify":
          case "notify.bottom":
@@ -1159,6 +1279,8 @@ var noscriptOverlay = noscriptUtil.service ?
     },
     
     webProgressListener: {
+      QueryInterface: noscriptUtil.service.generateQI([
+        Components.interfaces.nsIWebProgressListener]),
       STATE_STOP: Components.interfaces.nsIWebProgressListener.STATE_STOP,
       onLocationChange: function(aWebProgress, aRequest, aLocation) {
         const domWindow = aWebProgress.DOMWindow;
@@ -1186,7 +1308,7 @@ var noscriptOverlay = noscriptUtil.service ?
         var w = doc.defaultView;
         if (w) {
           const ns = noscriptOverlay.ns;
-          w.__NoScript_contentLoaded = true;
+          noscriptOverlay.ns.setExpando(w, "contentLoaded", true);
           if (w == w.top) {
             ns.processMetaRefresh(doc, noscriptOverlay.notifyMetaRefreshCallback);
             if (w == window.content) {
@@ -1220,6 +1342,7 @@ var noscriptOverlay = noscriptUtil.service ?
       try {
         noscriptOverlay.listeners.setup(); 
         noscriptOverlay.wrapBrowserAccess();
+        window.setTimeout(noscriptOverlay.pdfDownloadHack, 0);
       } catch(e) {
         var msg = "[NoScript] Error initializing new window " + e + "\n"; 
         noscriptOverlay.ns.log(msg);
@@ -1328,7 +1451,8 @@ var noscriptOverlay = noscriptUtil.service ?
     
     browserDOMWindow.wrappedJSObject.openURI = noscriptOverlay.browserAccess.openURI;
     
-    noscriptOverlay.ns.dump("[NoScript] browserDOMWindow wrapped for external load interception");
+    if(noscriptOverlay.ns.consoleDump) 
+      noscriptOverlay.ns.dump("[NoScript] browserDOMWindow wrapped for external load interception");
   },
   
   browserAccess: {
@@ -1366,9 +1490,17 @@ var noscriptOverlay = noscriptUtil.service ?
     }
   },
   
+  pdfDownloadHack: function() {
+    if (typeof(mouseClick) != "function") return;
+    var tb = getBrowser();
+    tb.removeEventListener("click", mouseClick, true);
+    tb.addEventListener("click", mouseClick, false);
+  },
+  
   install: function() {
     // this.ns.dump("*** OVERLAY INSTALL ***\n");
     this.ns.setPref("badInstall", false);
+    this.ns.domUtils._winType = document.documentElement.getAttribute("windowtype");
     window.addEventListener("load", this.listeners.onLoad, false);
   },
 
